
/* This structure holds global variables of the server */
struct serv
{
	int listening_socket;	/* File descriptor of the listening socket */
	int conn_limit;		/* Max # of simultaneous connections */
	int buff_limit;		/* Max amount of memory allocated for a single I/O buffer (1 connection = 2 buffers) */
	
	int conn_count;		/* Current number of connections */
	void *user_data;
	
	char *printf_buffer;
	
	struct conn * connections;	/* Linked list of connections */
	void * (*connect)(void*, struct serv *, struct conn *);
	void (*disconnect)(struct serv *, struct conn *, void *);
	int (*input)(struct serv*, struct conn*, void*, char *buf, int len);
};

/* This describes an I/O buffer */
struct conn_buffer
{
	unsigned int size;	/* Number of bytes kept under mem */
	unsigned int alloc;	/* Amount of memory allocated for this buffer (size <= alloc) */
	void *mem;		/* The actual data */
};

struct conn
{
	struct conn *next;	/* Next connection in the linked list */
	struct serv *server;

	int fd;			/* Socket */
	
	unsigned char state;	/* Connection state - see below */
	
	struct sockaddr_in peer;	/* Peer IP address/port */
	
	struct conn_buffer i;	/* Input buffer */
	struct conn_buffer o;	/* Output buffer */
	
	void *user_data; 
	
};

#define STATE_OPEN 0
#define STATE_CLOSING 1


void server_write_conn(struct serv *s, struct conn *c, void *ptr, size_t len);

int server_perform(struct serv *s);
void server_fdset(struct serv *s, fd_set *rd, fd_set *wr, fd_set *ex, int *maxfd);
void server_write(struct serv *s, void *ptr, size_t len);
void server_write_conn(struct serv *s, struct conn *c, void *ptr, size_t len);

struct serv * server_init(int port, char * address, int maxconn, int buffer_limit, void *,
		void * (*connect)(void *, struct serv *, struct conn *),
		void (*disconnect)(struct serv *, struct conn *, void *),
		int (*input)(struct serv*, struct conn*, void*, char *buf, int len));

void server_disconnect(struct conn *c);
void server_printf(struct serv *s, struct conn *c, char * fmt, ...);
